--
-- bigBagReceiver
-- Specialization for BigBagReceiver
--
-- @author John Deere 6930
-- @date 20/07/2020
--

--[[XML

	<bigBagReceiver fillUnitIndex="">
		<bigBagAttacherTrigger node="bigBagAttacherTrigger" />
		<iconAnimation animationName="" />
		<fill animationName="" />
	</bigBagReceiver>
]]

BigBagReceiver = {};
BigBagReceiver.modDir = g_currentModDirectory;
BigBagReceiver.MOD_NAME = g_currentModName;

function BigBagReceiver.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(FillUnit, specializations);
end;

function BigBagReceiver.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "bigBagAttacherTriggerCallback", 	BigBagReceiver.bigBagAttacherTriggerCallback);
    SpecializationUtil.registerFunction(vehicleType, "getIsObjectBigBagAttacher", 		BigBagReceiver.getIsObjectBigBagAttacher);
    SpecializationUtil.registerFunction(vehicleType, "bigBagRackIsEmpty", 				BigBagReceiver.bigBagRackIsEmpty);
    SpecializationUtil.registerFunction(vehicleType, "sendBigBagData", 					BigBagReceiver.sendBigBagData);
    SpecializationUtil.registerFunction(vehicleType, "setIconAnimation", 				BigBagReceiver.setIconAnimation);
    SpecializationUtil.registerFunction(vehicleType, "updateBigBagObjects", 			BigBagReceiver.updateBigBagObjects);
    SpecializationUtil.registerFunction(vehicleType, "updateBigBagRack", 				BigBagReceiver.updateBigBagRack);
end;

function BigBagReceiver.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getFillUnitSupportsToolType",	BigBagReceiver.getFillUnitSupportsToolType);
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getPrice",					    BigBagReceiver.getPrice);
end;

function BigBagReceiver.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", 					BigBagReceiver);
    SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", 				BigBagReceiver);
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", 					BigBagReceiver);
    SpecializationUtil.registerEventListener(vehicleType, "onReadStream", 				BigBagReceiver);
    SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", 				BigBagReceiver);
end

function BigBagReceiver:onLoad(savegame)
    self.spec_bigBagReceiver = self[string.format("spec_%s.bigBagReceiver", BigBagReceiver.MOD_NAME)]
    local spec = self.spec_bigBagReceiver
	
    spec.actionEvents = {};
	
	spec.pendingTriggerObjects = {};	
	spec.fillUnitIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.bigBagReceiver#fillUnitIndex"), 1);
	spec.currentBigBagAttacherObject = nil;
    if self.isServer then
        spec.bigBagAttacherTriggerId = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.bigBagReceiver.bigBagAttacherTrigger#node"), self.i3dMappings);
        if spec.bigBagAttacherTriggerId ~= nil then
            addTrigger(spec.bigBagAttacherTriggerId, "bigBagAttacherTriggerCallback", self);
        end;
    end;
	
    spec.iconAnimationName = getXMLString(self.xmlFile, "vehicle.bigBagReceiver.iconAnimation#animationName");
	if spec.iconAnimationName ~= nil then
		self:playAnimation(spec.iconAnimationName, -1, nil, true);
	end;
	
	spec.hasBigBagInRack = false;
    spec.fillAnimationName = getXMLString(self.xmlFile, "vehicle.bigBagReceiver.fill#animationName");
	
	spec.emptyBigBagRackPrice = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.bigBagReceiver#emptyBigBagRackPrice"), 500);
end;

function BigBagReceiver:onPostLoad(savegame)
	local spec = self.spec_bigBagReceiver;	
    local hasBigBagInRack = false;
    if savegame ~= nil and not savegame.resetVehicles then
        hasBigBagInRack = Utils.getNoNil(getXMLBool(savegame.xmlFile, string.format("%s.%s.bigBagReceiver#hasBigBagInRack", savegame.key, BigBagReceiver.MOD_NAME)), spec.hasBigBagInRack);		
    end;
    hasBigBagInRack = hasBigBagInRack or (self:getFillUnitFillLevel(spec.fillUnitIndex)>0);
	self:updateBigBagRack(hasBigBagInRack, true);
end;

function BigBagReceiver:saveToXMLFile(xmlFile, key, usedModNames)
    local spec = self.spec_bigBagReceiver;
    setXMLBool(xmlFile, key.."#hasBigBagInRack", spec.hasBigBagInRack);
end;

function BigBagReceiver:onDelete()
    local spec = self.spec_bigBagReceiver
    if spec.bigBagAttacherTriggerId ~= nil then
        removeTrigger(spec.bigBagAttacherTriggerId)
    end
	if spec.currentBigBagAttacherObject ~= nil then
		g_server:broadcastEvent(SetBigBagRackEvent:new(spec.currentBigBagAttacherObject, nil), true, nil, spec.currentBigBagAttacherObject);
		spec.currentBigBagAttacherObject = nil;
	end;
end

function BigBagReceiver:onReadStream(streamId, connection)
	local spec = self.spec_bigBagReceiver;	
    local hasBigBagInRack = streamReadBool(streamId);	
	self:updateBigBagRack(hasBigBagInRack, true);
end;

function BigBagReceiver:onWriteStream(streamId, connection)
	local spec = self.spec_bigBagReceiver;	
    streamWriteBool(streamId, spec.hasBigBagInRack);
end;

function BigBagReceiver:getFillUnitSupportsToolType(superFunc, fillUnitIndex, toolType)
    local spec = self.spec_bigBagReceiver;
	local retVal = false;
	if self:bigBagRackIsEmpty() then
		if toolType == ToolType.UNDEFINED then
			retVal = true;
		end;
	else
		retVal = superFunc(self, fillUnitIndex, toolType);
	end;
	return retVal;
end;

function BigBagReceiver:updateBigBagObjects()
    if self.isServer then
        local spec = self.spec_bigBagReceiver
		local currentObject = self:getIsObjectBigBagAttacher();
		if spec.currentBigBagAttacherObject ~= currentObject and currentObject ~= nil then
			if not currentObject:getHasBigBackRack() then
				spec.currentBigBagAttacherObject = currentObject;
				g_server:broadcastEvent(SetBigBagRackEvent:new(spec.currentBigBagAttacherObject, self), true, nil, spec.currentBigBagAttacherObject);
				self:setIconAnimation(true);
			end;
		end;
		if currentObject == nil and spec.currentBigBagAttacherObject ~= nil then
			g_server:broadcastEvent(SetBigBagRackEvent:new(spec.currentBigBagAttacherObject, nil), true, nil, spec.currentBigBagAttacherObject);
			spec.currentBigBagAttacherObject = nil;
			self:setIconAnimation(false);
		end;
    end;
end;

function BigBagReceiver:getIsObjectBigBagAttacher()
	local spec = self.spec_bigBagReceiver
	local bigBagAttacherObject = nil;
	for object,k in pairs(spec.pendingTriggerObjects) do
		if object ~= nil and object.getSupportsBigBag ~= nil and object:getSupportsBigBag() then
			bigBagAttacherObject = object;
			break;
		end;
	end;
    return bigBagAttacherObject;
end;

function BigBagReceiver:bigBagRackIsEmpty()
	local spec = self.spec_bigBagReceiver
	return not spec.hasBigBagInRack;
end;

function BigBagReceiver:setIconAnimation(state)
	local spec = self.spec_bigBagReceiver
	if spec.iconAnimationName ~= nil then
		if state then
			self:playAnimation(spec.iconAnimationName, 1, nil, false);
		else
			self:playAnimation(spec.iconAnimationName, -1, nil, false);
		end;
	end;
end;

function BigBagReceiver:updateBigBagRack(bigbagInRack, noEventSend)
	--Update BigBag visibility
	local spec = self.spec_bigBagReceiver;
	UpdateBigBagRackEvent.sendEvent(self, bigbagInRack, noEventSend);
	spec.hasBigBagInRack = bigbagInRack;			
	local direction = 1;
	if not bigbagInRack then
		direction = -1;
	end;
	self:playAnimation(spec.fillAnimationName, direction, nil, true);
end;

function BigBagReceiver:sendBigBagData(isReceiving, otherObject, noEventSend)
	SendBigBagDataEvent.sendEvent(self, isReceiving, otherObject, noEventSend);
	local spec = self.spec_bigBagReceiver
	if self.isServer and otherObject ~= nil then
		local farmId = self:getActiveFarm();
		if isReceiving then
			--BigBag in Rack
			local newFillType = otherObject:getFillUnitFillType(1);
			local newFillLevel = otherObject:getFillUnitFillLevel(1);	
			self:addFillUnitFillLevel(farmId, spec.fillUnitIndex, newFillLevel, newFillType, ToolType.UNDEFINED, nil);
			g_currentMission:removeVehicle(otherObject);
			self:updateBigBagRack(true);
		else
			--BigBag from Rack	
			local newBigBag = otherObject:createBigBag(true, false);
			if newBigBag ~= nil then
				local newFillType =  self:getFillUnitFillType(spec.fillUnitIndex);
				local newFillLevel = self:getFillUnitFillLevel(spec.fillUnitIndex);					
				newBigBag:addFillUnitFillLevel(farmId, 1, newFillLevel, newFillType, ToolType.UNDEFINED, nil);				
				self:addFillUnitFillLevel(farmId, spec.fillUnitIndex, -math.huge, newFillType, ToolType.UNDEFINED, nil);
				self:updateBigBagRack(false);
			end;
		end;
	end;
end;

function BigBagReceiver:bigBagAttacherTriggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
    local spec = self.spec_bigBagReceiver;
	if onEnter then
		local object = g_currentMission:getNodeObject(otherActorId);
        if object == nil then
            object = g_currentMission.nodeToObject[otherActorId];
        end;
        if object ~= nil and object ~= self and object.getSupportsMountDynamic ~= nil and object:getSupportsMountDynamic() then
            spec.pendingTriggerObjects[object] = Utils.getNoNil(spec.pendingTriggerObjects[object], 0) + 1;
        end;
    elseif onLeave then
        local object = g_currentMission:getNodeObject(otherActorId);
        if object == nil then
            object = g_currentMission.nodeToObject[otherActorId];
        end;
        if object ~= nil then
            if spec.pendingTriggerObjects[object] ~= nil then
				spec.pendingTriggerObjects[object] = math.max(0, spec.pendingTriggerObjects[object] - 1)
				if spec.pendingTriggerObjects[object] == 0 then
					spec.pendingTriggerObjects[object] = nil;
				end;
            end;
        end;
	end;
	self:updateBigBagObjects();
end;

-- Correct price-calc per FillLevel
function BigBagReceiver:getPrice(superFunc)
    local spec = self.spec_bigBagReceiver;
    local fillTypeIndex = self:getFillUnitFillType(spec.fillUnitIndex);
    local fillType = g_fillTypeManager:getFillTypeByIndex(fillTypeIndex);
    local pricePerLiter = fillType.pricePerLiter;
    local fillLevel = self:getFillUnitFillLevel(spec.fillUnitIndex);
    return spec.emptyBigBagRackPrice + fillLevel * pricePerLiter;
end;

