--
-- DischargeAnimation
-- Specialization for DischargeAnimations
--
-- @author John Deere 6930
-- @date 16/06/2019
--
DischargeAnimation = {};
DischargeAnimation.modDir = g_currentModDirectory;
DischargeAnimation.MOD_NAME = g_currentModName;

function DischargeAnimation.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations) and SpecializationUtil.hasSpecialization(Dischargeable, specializations)
end

function DischargeAnimation.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "loadDischargeAnimationFromXML", DischargeAnimation.loadDischargeAnimationFromXML)
end

function DischargeAnimation.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "setDischargeState", DischargeAnimation.setDischargeState)
end

function DischargeAnimation.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", DischargeAnimation)
end

function DischargeAnimation:onLoad(savegame)
    self.spec_dischargeAnimation = self[string.format("spec_%s.dischargeAnimation", DischargeAnimation.MOD_NAME)];
    local spec = self.spec_dischargeAnimation;
	spec.dischargeAnimations = {};
    i = 0;
    while true do
        local baseKey = string.format("vehicle.dischargeAnimations.dischargeAnimation(%d)", i)
        if not hasXMLProperty(self.xmlFile, baseKey) then
            break
        end;
        local entry = {};
        if self:loadDischargeAnimationFromXML(entry, self.xmlFile, baseKey) then
            table.insert(spec.dischargeAnimations, entry);
        end
        i = i + 1;
    end;
end;

function DischargeAnimation:loadDischargeAnimationFromXML(dischargeAnimation, xmlFile, key)
	dischargeAnimation.dischargeNodeIndex = getXMLInt(xmlFile, key .. "#dischargeNodeIndex");
    if dischargeAnimation.dischargeNodeIndex == nil then
        g_logManager:xmlWarning(self.configFileName, "Missing 'dischargeNodeIndex' for dischargeAnimation '%s'", key);
        return false;
    end
    dischargeAnimation.animationName = getXMLString(xmlFile, key.."#animationName");
    return dischargeAnimation.animationName ~= nil;
end;


function DischargeAnimation:setDischargeState(superFunc, state, noEventSend)
	superFunc(self, state, noEventSend);
	local currentDischargeNode = self.spec_dischargeable.currentDischargeNode;
    local spec = self.spec_dischargeAnimation;

	for _,dischargeAnim in pairs(spec.dischargeAnimations) do
		if  self.spec_dischargeable.dischargeNodes[dischargeAnim.dischargeNodeIndex] == currentDischargeNode then
			if state ~= Dischargeable.DISCHARGE_STATE_OFF then		
				self:playAnimation(dischargeAnim.animationName, 1, nil, true);
			else
				self:playAnimation(dischargeAnim.animationName, -1, nil, true);
			end;
		end;
	end;	
end;