--
-- SupportsBigBag
-- Specialization for SupportsBigBags
--
-- @author John Deere 6930
-- @date 30/04/2020
--

--[[XML

<supportsBigBag canCreateNewBigBags="">
	<attacher attacherJointIndex="" />	
	<creation bigBagSpawnNode="" filename="" isAttachedOnSpawn="" spawnedOnGround="" purchasingPrice="" />
</supportsBigBag>

]]

SupportsBigBag = {};
SupportsBigBag.modDir = g_currentModDirectory;
SupportsBigBag.MOD_NAME = g_currentModName;

function SupportsBigBag.prerequisitesPresent(specializations)
    return  SpecializationUtil.hasSpecialization(AttacherJoints, specializations);
end

function SupportsBigBag.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "getSupportsBigBag", 		SupportsBigBag.getSupportsBigBag);
    SpecializationUtil.registerFunction(vehicleType, "setBigBagRack", 			SupportsBigBag.setBigBagRack);	
    SpecializationUtil.registerFunction(vehicleType, "getHasBigBackRack", 		SupportsBigBag.getHasBigBackRack);
    SpecializationUtil.registerFunction(vehicleType, "hasAttachedBigBag", 		SupportsBigBag.hasAttachedBigBag);
    SpecializationUtil.registerFunction(vehicleType, "getAttachedBigBag", 		SupportsBigBag.getAttachedBigBag);
	SpecializationUtil.registerFunction(vehicleType, "createBigBag",            SupportsBigBag.createBigBag);
	SpecializationUtil.registerFunction(vehicleType, "getCanSpawnBigBag",     	SupportsBigBag.getCanSpawnBigBag);
	SpecializationUtil.registerFunction(vehicleType, "collisionTestCallback",   SupportsBigBag.collisionTestCallback);
end

function SupportsBigBag.registerEvents(vehicleType)
end

function SupportsBigBag.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", SupportsBigBag);
    SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", SupportsBigBag);
    SpecializationUtil.registerEventListener(vehicleType, "onDelete", SupportsBigBag);
end

function SupportsBigBag:onLoad(savegame)
    self.spec_supportsBigBag = self[string.format("spec_%s.supportsBigBag", SupportsBigBag.MOD_NAME)];
    local spec = self.spec_supportsBigBag;
    spec.actionEvents = {};	
	
	--attacher
	spec.canCreateNewBigBags = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.supportsBigBag#canCreateNewBigBags"), false);	
	spec.supportsBigBag = true;	
	spec.currentBigBagRack = nil;
	spec.attacherJointIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.supportsBigBag.attacher#attacherJointIndex"), 1);
	
	--creation
	local bigBagSpawnNode = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.supportsBigBag.creation#bigBagSpawnNode"), self.i3dMappings);
	if bigBagSpawnNode ~= nil then
		spec.bigBagSpawnNode = bigBagSpawnNode;	
		spec.bigBagAttacherJointIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.supportsBigBag.creation#bigBagAttacherJointIndex"), 1);
		spec.filename = getXMLString(self.xmlFile, "vehicle.supportsBigBag.creation#filename");
		spec.isAttachedOnSpawn = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.supportsBigBag.creation#isAttachedOnSpawn"), false);
		spec.spawnedOnGround = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.supportsBigBag.creation#spawnedOnGround"), true);
		spec.purchasingPrice = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.supportsBigBag.creation#purchasingPrice"), true);
	end;
	
    spec.lastBigBag = nil;
	spec.foundObjectAtSpawnPlace = false;
end;

function SupportsBigBag:getSupportsBigBag()
    local spec = self.spec_supportsBigBag;
	return spec.supportsBigBag; 
end;

function SupportsBigBag:onDelete()
    local spec = self.spec_supportsBigBag;
	if spec.currentBigBagRack ~= nil then
		g_currentMission:removeActivatableObject(spec.currentBigBagRack);
	end;
end;

function SupportsBigBag:setBigBagRack(bigBagRack)
    local spec = self.spec_supportsBigBag;
    spec.currentBigBagRack = bigBagRack;	
	SupportsBigBag.updateActionEvents(self);
end;

function SupportsBigBag:getHasBigBackRack()
    local spec = self.spec_supportsBigBag;
    return spec.currentBigBagRack ~= nil;
end;

function SupportsBigBag:hasAttachedBigBag()
	local spec = self.spec_supportsBigBag
    return (self:getImplementByJointDescIndex(spec.attacherJointIndex) ~= nil);
end;

function SupportsBigBag:getAttachedBigBag()
	local spec = self.spec_supportsBigBag
	local implement = self:getImplementByJointDescIndex(spec.attacherJointIndex)
    return implement.object;
end;

function SupportsBigBag:getCanSpawnBigBag()
    local spec = self.spec_supportsBigBag;
	local xmlFilename = Utils.getFilename(spec.filename, self.baseDirectory);
	local sizeWidth, sizeLength, _, _ = StoreItemUtil.getSizeValues(xmlFilename, "vehicle", 0);
	local x,y,z = getWorldTranslation(spec.bigBagSpawnNode);
	local rx,ry,rz = getWorldRotation(spec.bigBagSpawnNode);
	spec.foundObjectAtSpawnPlace = false;
	overlapBox(x, y, z, rx, ry, rz, sizeWidth*0.5, 0.1, sizeLength*0.5, "collisionTestCallback", self, 5468288);
	return not spec.foundObjectAtSpawnPlace;	
end;

function SupportsBigBag:collisionTestCallback(transformId)
    if g_currentMission.nodeToObject[transformId] ~= nil or g_currentMission.players[transformId] ~= nil or g_currentMission:getNodeObject(transformId) ~= nil then		
		local attacherVehicle = self:getAttacherVehicle();		
		--frontloader
		local frontloaderAttacherValid = true;
		if attacherVehicle.getAttacherVehicle ~= nil then
			local frontloaderAttacherVehicle = attacherVehicle:getAttacherVehicle();
			frontloaderAttacherValid = g_currentMission.nodeToObject[transformId] ~= frontloaderAttacherVehicle;
		end;		
		local terrainId = g_currentMission.terrainRootNode;
        if g_currentMission.nodeToObject[transformId] ~= self and g_currentMission.nodeToObject[transformId] ~= attacherVehicle and frontloaderAttacherValid or terrainId == transformId then
            self.spec_supportsBigBag.foundObjectAtSpawnPlace = true;
        end;
    end;
end;

function SupportsBigBag:createBigBag(forced, allowed, noEventSend)
    local spec = self.spec_supportsBigBag
	CreateBigBagEvent.sendEvent(self, forced, allowed, noEventSend);
	local bigBag = nil;
	if forced or allowed then
		if self.isServer then
			local farmId = self:getOwnerFarmId();
			local x, y, z = getWorldTranslation(spec.bigBagSpawnNode);
			local dirX, _, dirZ = localDirectionToWorld(spec.bigBagSpawnNode, 0, 1, 0)
			local yRot = MathUtil.getYRotationFromDirection(dirX, dirZ);				
			if spec.spawnedOnGround then
				y = nil;
			end;				
			local xmlFilename = Utils.getFilename(spec.filename, self.baseDirectory);
			bigBag = g_currentMission:loadVehicle(xmlFilename, x, y, z, 0, yRot, true, 0, Vehicle.PROPERTY_STATE_OWNED, farmId, nil, nil);
			if bigBag ~= nil then
				spec.lastBigBag = bigBag;
			end;		
			if spec.isAttachedOnSpawn then
				local attacherVehicle, attacherVehicleJointDescIndex, attachableBigBag, attachableBigBagJointDescIndex = AttacherJoints.findVehicleInAttachRange(self, AttacherJoints.MAX_ATTACH_DISTANCE_SQ, AttacherJoints.MAX_ATTACH_ANGLE);
				if attachableBigBag ~= nil then
					if attacherVehicle.spec_attacherJoints.attacherJoints[attacherVehicleJointDescIndex].jointIndex == 0 then
						attacherVehicle:attachImplement(attachableBigBag, attachableBigBagJointDescIndex, attacherVehicleJointDescIndex, true);
					end;
				end;
			end;			
			if spec.purchasingPrice and not forced then
				local costs = bigBag:getPrice();
				g_currentMission:farmStats(farmId):updateStats("expenses", costs);
				g_currentMission:addMoney(-costs, farmId, MoneyType.SHOP_VEHICLE_BUY);
				g_currentMission:addMoneyChange(-costs, farmId, MoneyType.SHOP_VEHICLE_BUY, true);
			end;		
		end;
		SupportsBigBag.updateActionEvents(self);
	end;		
	return bigBag;
end;

function SupportsBigBag:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
    if self.isClient then
        local spec = self.spec_supportsBigBag;
        self:clearActionEventsTable(spec.actionEvents);
        if isActiveForInputIgnoreSelection then
			local _, actionEventId = self:addActionEvent(spec.actionEvents, InputAction.IMPLEMENT_EXTRA, self, SupportsBigBag.actionEventBigBagRackReceiver, false, true, false, true, nil);
            g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL);
            SupportsBigBag.updateActionEvents(self);
			
			if spec.canCreateNewBigBags then
				_, actionEventId = self:addActionEvent(spec.actionEvents, InputAction.IMPLEMENT_EXTRA2, self, SupportsBigBag.actionEventBigBagCreation, false, true, false, true, nil);
				g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL);
				SupportsBigBag.updateActionEvents(self);
			end;
        end;
    end;
end;

function SupportsBigBag.actionEventBigBagRackReceiver(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_supportsBigBag;
	if self:getHasBigBackRack() then
		local bigBagRackIsEmpty = spec.currentBigBagRack:bigBagRackIsEmpty();
		if bigBagRackIsEmpty then
			if self:hasAttachedBigBag() then
				local bigBag = self:getAttachedBigBag();
				spec.currentBigBagRack:sendBigBagData(true, bigBag);
			end;
		else
			if not self:hasAttachedBigBag() then
				spec.currentBigBagRack:sendBigBagData(false, self);
			end;
		end;
	end;
end;

function SupportsBigBag.actionEventBigBagCreation(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_supportsBigBag;
	if self:getCanSpawnBigBag() and not self:hasAttachedBigBag() then	
		self:createBigBag(false, true);
	else
		g_currentMission:showBlinkingWarning(g_i18n:getText("warning_noFreeSpaceHere"), 2000);
	end;
end

function SupportsBigBag.updateActionEvents(self)
    local spec = self.spec_supportsBigBag;
    local actionEvent = spec.actionEvents[InputAction.IMPLEMENT_EXTRA];
    if actionEvent ~= nil then 
		local actionEventActive = false;
		if self:getHasBigBackRack() then
			local bigBagRackIsEmpty = spec.currentBigBagRack:bigBagRackIsEmpty();
			if bigBagRackIsEmpty then
				if self:hasAttachedBigBag() then
					g_inputBinding:setActionEventText(actionEvent.actionEventId, g_i18n:getText("action_hangBigBag"));
					actionEventActive = true;					
				end;
			else
				if not self:hasAttachedBigBag() then
					g_inputBinding:setActionEventText(actionEvent.actionEventId, g_i18n:getText("action_takeBigBag"));
					actionEventActive = true
				end;
			end;
		end;			
        g_inputBinding:setActionEventActive(actionEvent.actionEventId, actionEventActive)
    end;
	
	if spec.canCreateNewBigBags then
		actionEvent = spec.actionEvents[InputAction.IMPLEMENT_EXTRA2];
		if actionEvent ~= nil then
			local allowCreation = not self:hasAttachedBigBag() and not self:getHasBigBackRack();
			if allowCreation then
				g_inputBinding:setActionEventText(actionEvent.actionEventId, g_i18n:getText("action_buyBigBag"));
			end;
			g_inputBinding:setActionEventActive(actionEvent.actionEventId, allowCreation);
		end;
	end;
end;